;-- interr.asm ------------------------------------------------- >>fst'01<< -
; - Quelltext zum Interruptaufruf mit BASIC.
; - Aufruf in BASIC: CALL ABSOLUTE (AX%, BX%, CX%, DX%, ES%, Offset%)
; - Kann mit absolute.bas in einen String verwandelt werden.
; - Alle Zeichen, die mit + CHR$(xx) an den BASIC-String angefuegt wuerden,
;   werden durch entsprechende Gestaltung des Quelltextes vermieden.
; - Beispiel fuer die Benutzung in wait1583.bas
;-- V 20011006 ------------------------------ http://home.arcor.de/steini63 -

;-- Register sichern, die wir manipulieren und ------------------------------
;-- BP auf Spitze des Stacks zeigen lassen. ---------------------------------
;Leider kann man mit dem Stack-Pointer (SP) keine Speicher-Adressierung
;durchfuehren (z.B. [SP+2]). Deshalb muessen wir den Stack-Basepointer (BP)
;auf eine definierte Stelle setzen, naemlich die aktuelle Spitze des Stacks. 
;Anschliessend liegen alle von BASIC uebergebenen Parameter UEBER dem BP; 
;alles was nach "MOV BP,SP" auf des Stack geht DARUNTER (der Stack waechst 
;von oben nach unten!). 

	PUSH	BP		;originalen Basepointer auf Stack sichern
	MOV	BP,SP		;neuer Basepointer zeigt auf Spitze des Stacks
	PUSH	SI		;SI auf Stack sichern
	PUSH	ES		;ES auf Stack sichern

;-- Register laden mit den Werten der uebergebenen Variablen ----------------
;Auf dem Stack liegen die Speicheradressen (Offsets) der von BASIC ueber-
;gebenen Variablen jetzt ab BP+6 aufwaerts, beginnend mit der letzten 
;(rechtsbuendig neben Offset%) stehenden. Bei BP+4/BP+2 befinden sich 
;Segment/Offset der Ruecksprungadresse zu BASIC, bei BP+0 der gePUSHte Wert 
;des originalen BP.

	MOV	SI,[BP+6]	;Adresse des letzten Parameters nach SI
	MOV	ES,[SI]		;Wert des letzten Parameters nach ES
	MOV	SI,[BP+8]	;Adresse des vierten Parameters nach SI
	MOV	DX,[SI]		;Wert des vierten Parameters nach DX
	INC	BP		;BP manupulieren, um ASCII 10 zu vermeiden
	MOV	SI,[BP+9]	;Adresse des dritten Parameters nach SI
	DEC	BP		;BP wiederherstellen
	MOV	CX,[SI]		;Wert des dritten Parameters nach CX
	MOV	SI,[BP+12]	;Adresse des zweiten Parameters nach SI
	MOV	BX,[SI]		;Wert des zweiten Parameters nach BX
	MOV	SI,[BP+14]	;Adresse des ersten Parameters nach SI
	MOV	AX,[SI]		;Wert des ersten Parameters nach AX

;-- Interrupt aufrufen ------------------------------------------------------ 
;Hinter INT muss eine Zahl (Konstante) folgen, ein Registerinhalt oder eine
;Adresse ist unzulaessig. Deshalb wird der Wert vor Aufruf von CALL ABSOLUTE
;in Basic ueberschrieben (z.B. mit POKE ...). 

	INT	16		;beliebige Vorgabe: Interrupt 10H: BIOS

;-- Variablen mit den Registerwerten ueberschreiben ------------------------- 
;Manche Interrupts schreiben Rueckgabewerte in die verschiedenen Register.
;Alle uebergebenen Variablen werden deshalb mit den aktuellen Registerwerten
;ueberschrieben. In Basic kann der (ggf. neue) Wert ausgelesen werden.

	MOV	SI,[BP+6]	;Adresse des letzten Parameters nach SI
	MOV	[SI],ES		;Variable mit Wert aus ES ueberschreiben
	MOV	SI,[BP+8]	;Adresse des letzten Parameters nach SI
	MOV	[SI],DX		;Variable mit Wert aus DX ueberschreiben
	INC	BP		;BP manupulieren, um ASCII 10 zu vermeiden
	MOV	SI,[BP+9]	;Adresse des letzten Parameters nach SI
	DEC	BP		;BP wiederherstellen
	MOV	[SI],CX		;Variable mit Wert aus CX ueberschreiben
	MOV	SI,[BP+12]	;Adresse des letzten Parameters nach SI
	MOV	[SI],BX		;Variable mit Wert aus BX ueberschreiben
	MOV	SI,[BP+14]	;Adresse des letzten Parameters nach SI
	MOV	[SI],AX 	;Variable mit Wert aus AX ueberschreiben

;-- Zurueck zu Basic --------------------------------------------------------
;Statt "RETF  10"  benutzen wir eine alternative Routine. Wir wollen den 
;Maschinencode in einem String speichern. Die Zeichen Nr. 0, 10, 13 und 34
;machen in der Programmierumgebung (Editor) Probleme , die Zeichen Nr. 1, 2
;und 26 dem Compiler (mit + CHR$(xx) geht es aber auch).

	MOV	AX, [BP+4]	;Segment der Ruecksprungadresse sichern
	DEC	BP		;BP manupulieren, um ASCII 2 zu vermeiden
	MOV	BX, [BP+3]	;Offset  der Ruecksprungadresse sichern
	INC	BP		;BP wiederherstellen
	POP	ES		;ES wieder auf alten Stand
	POP	SI		;SI wieder auf alten Stand
	POP	BP		;Basepointer wieder auf alten Stand
	SUB	SP,14		;Stackpointer 14 Byte zuruecksetzen (4 Byte
				;Ruecksprungadr. + 10 Byte fuer 5 Parameter)
	PUSH	AX		;Segment der Ruecksprungadresse auf Stack
	PUSH	BX		;Offset  der Ruecksprungadresse auf Stack
	RETF			;zurueck zu Basic